==> zeitwerk-2.7.2.tar.gz: zeitwerk-2.7.2/test/lib/zeitwerk/test_unregister.rb <==
# frozen_string_literal: true

require "test_helper"

class TestUnregister < LoaderTest
  test "unregister removes the loader from internal state" do
    loader1 = Zeitwerk::Loader.new
    cref1 = Zeitwerk::Cref.new(Object, :"dummy1")
    registry = Zeitwerk::Registry
    registry.register_loader(loader1)
    registry.gem_loaders_by_root_file["dummy1"] = loader1
    registry.register_autoload(loader1, "dummy1")
    Zeitwerk::Registry::ExplicitNamespaces.__register(cref1, loader1)

    loader2 = Zeitwerk::Loader.new
    cref2 = Zeitwerk::Cref.new(Object, :"dummy2")
    registry = Zeitwerk::Registry
    registry.register_loader(loader2)
    registry.gem_loaders_by_root_file["dummy2"] = loader2
    registry.register_autoload(loader2, "dummy2")
    Zeitwerk::Registry::ExplicitNamespaces.__register(cref2, loader2)

    loader1.unregister

    assert !registry.loaders.include?(loader1)
    assert !registry.gem_loaders_by_root_file.values.include?(loader1)
    assert !registry.autoloads.values.include?(loader1)
    assert !Zeitwerk::Registry::ExplicitNamespaces.__registered?(cref1)

    assert registry.loaders.include?(loader2)
    assert registry.gem_loaders_by_root_file.values.include?(loader2)
    assert registry.autoloads.values.include?(loader2)
    assert Zeitwerk::Registry::ExplicitNamespaces.__registered?(cref2) == loader2
  end

  test 'with_loader yields and unregisters' do
    loader = Zeitwerk::Loader.new
    unregister_was_called = false
    loader.define_singleton_method(:unregister) { unregister_was_called = true }

    Zeitwerk::Loader.stub :new, loader do
      Zeitwerk.with_loader do |l|
        assert_same loader, l
      end
    end

    assert unregister_was_called
  end

  test 'with_loader yields and unregisters, even if an exception happens' do
    loader = Zeitwerk::Loader.new
    unregister_was_called = false
    loader.define_singleton_method(:unregister) { unregister_was_called = true }

    Zeitwerk::Loader.stub :new, loader do
      Zeitwerk.with_loader { raise } rescue nil
    end

    assert unregister_was_called
  end
end

==> zeitwerk-2.7.3.tar.gz: zeitwerk-2.7.3/test/lib/zeitwerk/test_unregister.rb <==
# frozen_string_literal: true

require "test_helper"

class TestUnregister < LoaderTest
  test "unregister removes the loader from internal state" do
    loader1 = Zeitwerk::Loader.new
    cref1 = Zeitwerk::Cref.new(Object, :"dummy1")
    registry = Zeitwerk::Registry
    registry.loaders.register(loader1)
    registry.gem_loaders_by_root_file["dummy1"] = loader1
    registry.autoloads.register("dummy1", loader1)
    registry.explicit_namespaces.register(cref1, loader1)

    loader2 = Zeitwerk::Loader.new
    cref2 = Zeitwerk::Cref.new(Object, :"dummy2")
    registry = Zeitwerk::Registry
    registry.loaders.register(loader2)
    registry.gem_loaders_by_root_file["dummy2"] = loader2
    registry.autoloads.register("dummy2", loader2)
    registry.explicit_namespaces.register(cref2, loader2)

    loader1.unregister

    assert !registry.loaders.registered?(loader1)
    assert !registry.gem_loaders_by_root_file.values.include?(loader1)
    assert !registry.autoloads.registered?("dummy1")
    assert !registry.explicit_namespaces.registered?(cref1)

    assert registry.loaders.registered?(loader2)
    assert registry.gem_loaders_by_root_file.values.include?(loader2)
    assert registry.autoloads.registered?("dummy2")
    assert registry.explicit_namespaces.registered?(cref2) == loader2
  end

  test 'with_loader yields and unregisters' do
    loader = Zeitwerk::Loader.new
    unregister_was_called = false
    loader.define_singleton_method(:unregister) { unregister_was_called = true }

    Zeitwerk::Loader.stub :new, loader do
      Zeitwerk.with_loader do |l|
        assert_same loader, l
      end
    end

    assert unregister_was_called
  end

  test 'with_loader yields and unregisters, even if an exception happens' do
    loader = Zeitwerk::Loader.new
    unregister_was_called = false
    loader.define_singleton_method(:unregister) { unregister_was_called = true }

    Zeitwerk::Loader.stub :new, loader do
      Zeitwerk.with_loader { raise } rescue nil
    end

    assert unregister_was_called
  end
end
